package com.miaxis.pc.controller;

import com.miaxis.common.core.controller.BaseController;
import com.miaxis.common.core.domain.Response;
import com.miaxis.common.core.page.ResponsePageInfo;
import com.miaxis.order.domain.OrderInfo;
import com.miaxis.order.dto.OrderInfoDto;
import com.miaxis.order.service.IOrderInfoService;
import com.miaxis.wx.domain.WxOrder;
import com.miaxis.wx.service.IRefundRecordService;
import com.miaxis.wx.service.IWxOrderService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

import static com.miaxis.common.utils.OrderCodeFactory.getOrderCode;

/**
 * 【订单信息】Controller
 *
 * @author miaxis
 * @date 2022-06-07
 */
@RestController
@RequestMapping("/order/info")
@Api(tags={"【PC-订单信息】"})
public class PcOrderInfoController extends BaseController {
    @Autowired
    private IOrderInfoService orderInfoService;
    @Autowired
    private IWxOrderService wxOrderService;

    @Autowired
    private IRefundRecordService refundRecordService;

    /**
     * 查询订单信息列表
     */
    @GetMapping("/list")
    @ApiOperation("查询订单信息列表")
        @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNum",value = "当前页码" ,dataType = "int", paramType = "query", required = false),
            @ApiImplicitParam(name = "pageSize",value = "每页数据量" , dataType = "int", paramType = "query", required = false),
    })
    public ResponsePageInfo<OrderInfo> list(@ModelAttribute OrderInfoDto orderInfoDto){
        startPage();
        List<OrderInfo> list = orderInfoService.selectOrderInfoList(orderInfoDto);
        return toResponsePageInfo(list);
    }

    /**
     * 发起分账
     */
    @PutMapping(value = "/shareProfit/{outTradeNo}")
    @ApiOperation("发起分账")
    public Response<OrderInfo> shareProfit(
            @ApiParam(name = "outTradeNo", value = "商户订单信息", required = true)
            @PathVariable("outTradeNo") String outTradeNo) throws Exception {
        OrderInfo orderInfo = orderInfoService.getByOutTradeNo(outTradeNo);

        if (orderInfo.getTradeType()!=1) {
            Response response = new Response(200, "非支付成功的订单，无法分账。");
            return response;
        }
        if (orderInfo.getProfitSharing() != 1) {
            Response response = new Response(200, "非待分账订单，无法分账。");
            return response;
        }

        // 获取当前时间
        Date currentDate = new Date();
        // 添加5分钟
        long fiveMinutesInMillis = 1 * 60 * 1000; // 5分钟的毫秒数
        Date nowDate = new Date(currentDate.getTime() - fiveMinutesInMillis);

        if (nowDate.compareTo(orderInfo.getSuccessTime()) < 0) {
            Response response = new Response(200, "支付成功的订单，未满5分钟无法分账，请稍后操作。");
            return response;
        }

        boolean addFlag = wxOrderService.wxAddFenZhang(orderInfo);
        boolean flag = wxOrderService.wxProfitsharing(orderInfo);
        if (flag && addFlag) {
            Response response = new Response(200, "分账成功");
            return response;
        } else {
            Response response = new Response(200, "分账失败，请联系管理员");
            return response;
        }

    }



    /**
     * 获取订单信息详细信息
     */
    @GetMapping(value = "/{id}")
    @ApiOperation("获取订单信息详细信息")
    public Response<OrderInfo> getInfo(
            @ApiParam(name = "id", value = "订单信息参数", required = true)
            @PathVariable("id") Long id
    ){
        return Response.success(orderInfoService.getById(id));
    }


    /**
     * 发起退款
     */
    @PutMapping(value = "/refund/{outTradeNo}")
    @ApiOperation("发起退款")
    public Response<OrderInfo> refund(
            @ApiParam(name = "outTradeNo", value = "订单信息参数", required = true)
            @PathVariable("outTradeNo") String outTradeNo,
            String reason
    ) throws Exception {
        String refundCode = getOrderCode(null);
        WxOrder order = wxOrderService.getByOutTradeNo(outTradeNo);
        refundRecordService.refund(order,refundCode,reason);
        return Response.success();
    }





}
